#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#       主応力の計算
#


import math

#
#  pricpalStress
#    主応力の計算
#    入力:[xx, yy, zz, xy, yz, zx]
#    戻り:[sigma1, sigma2, sigma3]
def principalStress(sigmaList):
    #xx,   yy,   zz,   xy,   yz,   zx
    [SG11, SG22, SG33, SG12, SG23, SG13] = sigmaList
    #不変量
    StInv1 = SG11 + SG22 + SG33
    StInv2 = (-(SG11 * SG22 + SG22 * SG33 + SG33 * SG11) + SG12*SG12 + SG13*SG13 +
                SG23*SG23)
    StInv3 = (SG11 * SG22 * SG33 + 2.0 * SG12 * SG23 * SG13
                - (SG11 * SG23*SG23 + SG22 * SG13*SG13 + SG33 * SG12*SG12))

    a = 1.0
    b = -StInv1
    c = -StInv2
    d = -StInv3
    p = (3.0 * a * c - b * b) / (9.0 * a*a)
    q = (2.0 * b*b*b - 9.0 * a * b * c + 27.0 * a*a * d) / (27.0 * a*a*a)
    DDD = -(4.0 * p*p*p + q*q)

    Sigm1 = 0.0
    Sigm2 = 0.0
    Sigm3 = 0.0

    if (DDD > 0.0):
        AlpRe = -0.5 * q
        AlpIm = 0.5 * pow(DDD, 0.5)
        R = pow((AlpRe*AlpRe + AlpIm*AlpIm), 0.5)
        if AlpRe == 0.0:
            Theta = 3.1415923238/2.0 * (AlpIm/abs(AlpIm))
        else:
            Theta = math.atan(AlpIm / AlpRe)
        R3 = pow(R, 0.3333)
        Theta3 = Theta / 3.0
        Del1 = R3 * math.cos(Theta3)
        Del2 = R3 * math.sin(Theta3)
        if Theta < 0:
            Del1=-Del1
            Del2=-Del2

        kushi1 = 2.0 * Del1
        kushi2 = -Del1 - 1.7321 * Del2
        kushi3 = -Del1 + 1.7321 * Del2

        SS1 = kushi1 - b / (3.0 * a)
        SS2 = kushi2 - b / (3.0 * a)
        SS3 = kushi3 - b / (3.0 * a)

        if SS1 > SS2 and SS2 > SS3:
            Sigm1 = SS1
            Sigm2 = SS2
            Sigm3 = SS3
        elif SS1 > SS2 and SS3 > SS2 and SS1 > SS3:
            Sigm1 = SS1
            Sigm2 = SS3
            Sigm3 = SS2
        elif SS2 > SS1 and SS2 > SS3 and SS1 > SS3:
            Sigm1 = SS2
            Sigm2 = SS1
            Sigm3 = SS3
        elif SS2 > SS1 and SS2 > SS3 and SS3 > SS1:
            Sigm1 = SS2
            Sigm2 = SS3
            Sigm3 = SS1
        elif SS2 > SS1 and SS3 > SS2:
            Sigm1 = SS3
            Sigm2 = SS2
            Sigm3 = SS1
    return [Sigm1, Sigm2, Sigm3]


if __name__ == "__main__":
    #xx=4.61E+02; yy=1.50E+02; zz=2.07E+02
    #xy=-3.13E+01; yz=-1.15E+01; zx=-6.62E+01
    xx=4.61E+02; yy=1.50E+02; zz=2.07E+02
    xy=-3.13E+01; yz=1.15E+01; zx=6.62E+01

    sigmaList = [xx, yy, zz, xy, yz, zx]
    [sig1, sig2, sig3] = principalStress(sigmaList)
    mises2 = (pow((sig1-sig2),2) + pow((sig2-sig3),2) + pow((sig3-sig1),2))/2.0
    mises = pow(mises2, 0.5)
    print sig1, sig2, sig3, mises

