#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
#
#       pyFistr.py
#
#   15/09/02    windows対応の為、"\n"→os.linesepに修正
#
#

import abaqus2fistr
import calcPrincipalStress
import csv
import os

ls = os.linesep             #行末コード

#------ data 変換--------------------
#
#  getElementTypeFistr2inp
#    fistrの要素タイプをinp用に変換
def getElementTypeFistr2inp(nameType):
    data = [["341", "tet"], ["342", "tet2"], ["361", "hex"], ["362", "hex2"],
            ["351", "prism"], ["352", "prism2"]]
    for names in data:
        if names[0] == nameType:
            inpType = names[1]
    return inpType

#
#  getNodesOrderOfElement
#    faceNoからelementのnodeNoを取得
#    nodeNoの並びは、fistrの並び順
def getNodesOrderOfElement(elType, faceNo):
    data = [["tet", [[1,2,3], [1,2,4], [2,3,4], [1,3,4]]],
            ["tet2", [[1,2,3], [1,2,4], [2,3,4], [1,3,4]]],
            ["hex", [[1,2,3,4], [5,6,7,8], [1,2,5,6], [2,3,6,7], [3,4,7,8],
                     [1,4,5,8]]],
            ["hex2", [[1,2,3,4], [5,6,7,8], [1,2,5,6], [2,3,6,7], [3,4,7,8],
                     [1,4,5,8]]],
            ["prism", [[1,2,3], [4,5,6], [1,2,4,5], [2,3,5,6], [1,3,4,6]]],
            ["prism2", [[1,2,3], [4,5,6], [1,2,4,5], [2,3,5,6], [1,3,4,6]]]]
    nodes = []
    for typeFaceNo in data:
        if typeFaceNo[0] == elType:
            nodes = typeFaceNo[1][faceNo-1]
            break
    return nodes

#
#  changeMeshScale
#    meshのscaleを変更
def changeMeshScale(lines, scale):
    i = getNextHeaderName(lines, 0, "!NODE", "")
    i += 1
    ps = i
    newNodes = []
    while lines[i][0] != "!":
        words = lines[i].split(",")
        words[1] = str(float(words[1]) * scale)
        words[2] = str(float(words[2]) * scale)
        words[3] = str(float(words[3]) * scale)
        newLine = ", ".join(words) + ls
        newNodes.append(newLine)
        i += 1
    newLines = lines[:ps] + newNodes + lines[i:]
    return newLines

#
#  getNodesFromElementSurface
#    elementとfaceNoからnodesを取得
def getNodesFromElementSurface(elements, elementNo, faceNo):
    elType = elements[elementNo][0]
    nodesAll = elements[elementNo][1:]
    nodesStd = getNodesOrderOfElement(elType, faceNo)
    nodes = []
    for node in nodesStd:
        nodes.append(nodesAll[node-1])
    return nodes

#
#  getSurfaceGroupsInLines
#    surfaceGroupを取得
def getSurfaceGroupsInLines(lines):
    surGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!SGROUP", "")
        if i < len(lines):
            i += 1
            [surs, i] = getOnlyDataInLines(lines, i)
            surGrp.append(surs)
    return surGrp

#
#  getNodeGroupsInLines
#    nodeGroupを取得
def getNodeGroupsInLines(lines):
    nodeGrp = []
    i = 0
    while i < len(lines):
        i = getNextHeaderName(lines, i, "!NGROUP", "")
        if i < len(lines):
            i += 1
            [nodes, i] = getOnlyDataInLines(lines, i)
            nodeGrp.append(nodes)
    return nodeGrp

#  getOnlyDataInLines
#    指定行からdataのみ取得
def getOnlyDataInLines(lines, i):
    data = []
    while i < len(lines):
        if lines[i][0] != "!":
            if lines[i][0] != "#":
                words = deleteSp(lines[i]).split(",")
                for word in words:
                    if word != "":
                        data += [word]
        else:
            break
        i += 1
    return [data, i]

#  deleteOnlyDataInLines
#    指定行からdataのみ削除
def deleteOnlyDataInLines(lines, i):
    ip = i
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            break
        i += 1
    newLines = lines[:ip] + lines[i:]
    return [newLines, ip]

#
#  getMeshDataForParaview
#    paraview用のmeshData(nodeとelement）を取得
def getMeshDataForParaview(fileName):
    f=open(fileName); lines=f.readlines(); f.close()
    nNode = int(lines[3].split()[0])
    nElement = int(lines[3].split()[1])
    meshLines = lines[4:nNode+nElement+4]
    return [meshLines, nNode, nElement]

#
#  addMeshDataConvVisData
#    fistr用結果dataをvis用に変換
def addMeshDataConvVisData(lines, meshData):
    #結果データを成形
    newLine = []
    i = 1
    nNode = int(lines[i].split()[0])
    nElement = int(lines[i].split()[1])
    i += 1
    nNodeVal = int(lines[i].split()[0])
    nElementVal = int(lines[i].split()[1])
    i += 1
    #nodeのデータ取得
    nodeValName = lines[i:i+nNodeVal+1]
    words = lines[i].split()
    nValue = 0
    for word in words:
        nValue += int(word)
    i += nNodeVal + 1
    newNodeVal = []
    n = 0
    while n < nNode:
        No = lines[i].split()[0]
        line = No + " "
        i += 1
        nVal = 0
        while nVal < nValue:
            subLine = lines[i].split(ls)[0]
            line += subLine + " "
            nVal += len(subLine.split())
            i += 1
        newNodeVal.append(line + ls)
        n += 1
    #elementのデータを取得
    if i < len(lines):
        elementValName = lines[i:i+nElementVal+1]
        nValue = 0
        words = lines[i].split()
        for word in words:
            nValue += int(word)
        i += nElementVal + 1
        newElementVal = []
        n = 0
        while n < nElement:
            No = lines[i].split()[0]
            line = No + " "
            i += 1
            nVal = 0
            while nVal < nValue:
                subLine = lines[i].split(ls)[0]
                line += subLine + " "
                nVal += len(subLine.split())
                i += 1
            newElementVal.append(line + ls)
            n += 1
    else:
        elementValName = []
        newElementVal = []
    #変数名の修正
    nodeValName[0] = str(nNodeVal) + " " + " ".join(nodeValName[0].split()) + ls
    i = 0
    while i < nNodeVal:
        nodeValName[i+1] = nodeValName[i+1].split(ls)[0] + ", unit_unknown" + ls
        i += 1
    if len(elementValName) != 0:
        elementValName[0] = (str(nElementVal) + " " +
                " ".join(elementValName[0].split()) + ls)
        i = 0
        while i < nElementVal:
            elementValName[i+1] = (elementValName[i+1].split(ls)[0] +
                ", unit_unknown" + ls)
            i += 1
    #newData作成
    newLines = [" ".join([str(nNode), str(nElement), str(nNodeVal), str(nElementVal), str(nNodeVal+nElementVal)]) + ls]
    newLines += meshData
    newLines += nodeValName
    newLines += newNodeVal
    newLines += elementValName
    newLines += newElementVal
    return newLines

#
#  readCsvData
#    csvデータの読み込み a☓bのtableDataを読み込む
def readCsvData(fileName):
    #tab区切りorカンマ区切りのチェック
    #  1行読み込み
    f=open(fileName); line=f.readline(); f.close()
    if len(line.split("\t")) != 1:
        #tab区切り
        f=open(fileName); tabCont=f.readlines(); f.close()
        contents = []
        for line in tabCont:
            contents.append(",".join(line.split("\t")))
        cont = csv.reader(contents)
    else:
        #カンマ区切り
        cont = csv.reader(file(fileName))
    data = []
    for row in cont:
        if len(row) > 0:
            data.append(row)
    return data

#
#  isPrincipalStressInVtk
#    結果fileに主応力が追加されているかどうかチェック
def isPrincipalStressInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("maxSigma1")
        if n >= 0:
            flag = True
            break
    return flag

#
#  isNodalStressInVtk
#    結果fileに応力tensorがあるかチェック
def isNodalStressInVtk(lines):
    flag = False
    for line in lines:
        n = line.find("NodalSTRESS")
        if n >= 0:
            flag = True
            break
    return flag

#
#  addPrincipalStressDataInVtk
#    主応力を結果fileに追加
def addPrincipalStressDataInVtk(lines):
    #FIELD数を修正
    i = 0
    while i < len(lines):
        n = lines[i].find("POINT_DATA ")
        if n >= 0:
            nn = -1
            while i < len(lines):
                nn = lines[i].find("FIELD ")
                if nn >= 0:
                    words = lines[i].split()
                    words[2] = str(int(words[2]) + 3)
                    lines[i] = " ".join(words) + ls
                    break
                i += 1
            if nn >= 0:
                break
        i += 1
    i += 1
    insertP = i
    #主応力を計算
    addData = [[], [], []]
    while i < len(lines):
        n = lines[i].find("NodalSTRESS ")
        if n >= 0:
            nNodes = int(lines[i].split()[2])
            i += 1
            ip = i
            while i < ip+nNodes:
                words = lines[i].split()
                stressList = []
                for word in words:
                    stressList.append(float(word))
                [sig1, sig2, sig3] = calcPrincipalStress.principalStress(stressList)
                addData[0].append(str(sig1))
                addData[1].append(str(sig2))
                addData[2].append(str(sig3))
                i += 1
        if len(addData[0]) > 0:
            break
        i += 1
    #追加する行を作成
    addLines = ["maxSigma1 1 " + str(nNodes) + " float" + ls]
    for val in addData[0]:
        addLines += [str(val) + ls]
    addLines += ["maxSigma2 1 " + str(nNodes) + " float" + ls]
    for val in addData[1]:
        addLines += [str(val) + ls]
    addLines += ["maxSigma3 1 " + str(nNodes) + " float" + ls]
    for val in addData[2]:
        addLines += [str(val) + ls]
    #主応力を追加
    lines = lines[:insertP] + addLines + lines[insertP:]
    return lines

#
#  addPrincipalStressData
#    結果データに主応力を追加する
def addPrincipalStressData(lines):
    #node数、element数を取得
    words = lines[0].split()
    nNodes = int(words[0])
    nElements = int(words[1])
    #NodalSTRESSを確認
    i = nNodes + nElements + 1
    nParas = lines[i].split()
    nVal = int(nParas[0])
    i += 1
    ip = i
    col = 1
    flag = "no"
    while i < ip + nVal:
        valName = deleteSp(lines[i]).split(",")[0]
        if valName == "NodalSTRESS":
            flag = "yes"
            break
        col += int(nParas[i-ip+1])
        i += 1
    if flag == "no":
        return lines

    #主応力を追加
    newLines = []
    #1行目を修正
    words = lines[0].split()
    words[2] = str(int(words[2]) + 3)
    words[4] = str(int(words[4]) + 3)
    lines[0] = " ".join(words) + ls
    #meshデータの次行を修正
    words = lines[ip-1].split()
    words[0] = str(int(words[0]) + 3)
    lines[ip-1] = " ".join(words) + " 1 1 1" + ls
    #変数を追加
    addLines =  ["maxSigma1, unit_unknown" + ls]
    addLines += ["midSigma2, unit_unknown" + ls]
    addLines += ["minSigma3, unit_unknown" + ls]
    newLines = lines[:ip+nVal] + addLines
    i = ip + nVal
    #while i < len(lines):
    n = 0
    while n < nNodes:
        stressWords = lines[i].split()[col:col+6]
        stressList = []
        for word in stressWords:
            stressList.append(float(word))
        [sig1, sig2, sig3] = calcPrincipalStress.principalStress(stressList)
        words = [str(sig1), str(sig2), str(sig3)]
        addWords = " ".join(words)
        lines[i] = lines[i].split(ls)[0]+ " " + addWords + ls
        n += 1
        i += 1
    newLines += lines[ip+nVal:]
    return newLines

#  findNumberFromList
#    挟み撃ち法でlist中からnewNoを返す
#      入力:[[oldNo, newNo]...], No
#      戻り:newNo
def findNumberFromList(nodes, No):
    newNode = "0"
    if len(nodes) > 10:
        i = int(len(nodes)/2)
        node = nodes[i][0]
        if int(No) < int(node):
            newNodes = nodes[:i]
            newNode = findNumberFromList(newNodes, No)
            return newNode
        else:
            newNodes = nodes[i:]
            newNode = findNumberFromList(newNodes, No)
            return newNode
    else:
        for node in nodes:
            if node[0] == No:
                newNode = node[1]
                break
        return newNode


#
#  renumberMeshData
#    meshDataのみをrenumberする
def renumberMeshData(meshData, nNode, nElement):
    nodeData = meshData[:nNode]
    elementData = meshData[nNode:]
    #tmpデータ（list）を作成
    tmpNodes = []
    i = 1
    for line in nodeData:
        words = line.split()
        nodeNo = int(words[0])
        tmpNodes.append([nodeNo, i])
        nodeData[i-1] = str(i) + " " + " ".join(words[1:]) + ls
        i += 1
    maxNodeNo = max(tmpNodes)[0]
    nodeNoList = []
    #配列を準備
    i = 0
    while i < maxNodeNo+1:
        nodeNoList.append("")
        i += 1
    #list作成
    for [nodeNo, newNo] in tmpNodes:
        nodeNoList[nodeNo] = newNo
    #elementを修正
    i = 1
    for line in elementData:
        words = line.split()
        words[0] = str(i)
        newList = []
        for node in words[3:]:
            n = int(node)
            newNode = str(nodeNoList[n])
            newList.append(newNode)
        lineList = words[:3] + newList
        elementData[i-1] = " ".join(lineList) + ls
        i += 1
    newMeshData = nodeData + elementData
    return newMeshData

#
#  renumberRssultDataForParaview
#    node, elementのNoを修正する。
def renumberResultDataForParaview(lines):
    words = lines[0].split()
    nNode = int(words[0])
    nElem = int(words[1])
    #meshDataがrenumberされているかチェック
    if (nNode == int(lines[nNode].split()[0]) and
                nElem == int(lines[nNode+nElem].split()[0])):
        pass
    else:
        #meshDataをrenumberする
        meshLines = lines[1:1+nNode+nElem]
        meshLines = renumberMeshData(meshLines, nNode, nElem)
        lines = lines[:1] + meshLines + lines[1+nNode+nElem:]
    #nodeデータを修正
    i = nNode + nElem + 1
    words = lines[i].split()
    ir = i + int(words[0]) + 1
    i = 0
    while i < nNode:
        line = lines[i+ir]
        words = line.split()
        lines[i+ir] = str(i+1) + " " + " ".join(words[1:]) + ls
        i += 1
    #elementデータを修正
    ip = i + ir
    if ip < len(lines):
        nVal = int(lines[ip].split()[0])
        ir = ip + nVal + 1
        n = 0
        while n < nElem:
            words = lines[ir+n].split()
            lines[ir+n] = str(n+1) + " " + " ".join(words[1:]) + ls
            n += 1
    return lines

#
#  convertResultDataForParaview
#    UCD formatを旧のformatに変換する
def convertResultDataForParaview(lines):
    #3行skip
    lines = lines[3:]
    #node数、element数を取得
    words = lines[0].split()
    nNodes = int(words[0])
    nElements = int(words[1])
    #変換する全行を取得（要素座標直後の行を削除）
    i = nNodes + nElements
    newLines = lines[:i+1] + lines[i+2:]
    #出力変数の数を取得
    while i < len(lines):
        if lines[i][0] >= "A":
            break
        i += 1
    nVal = 0
    while i < len(lines):
        if "A" <= lines[i][0]:
            nVal += 1
        else:
            break
        i += 1
    #1行目を修正（節点の変数の数、要素の変数の数、モデルの変数の数）
    words = newLines[0].split()
    words += [str(nVal), "0", str(nVal)]
    newLines[0] = " ".join(words) + ls
    return newLines    

#
#  getInsertPointInCnt
#    lineを追加する時の挿入場所を取得
#    keywordsの次の行No、存在しない場合はheader(!DLOAD等）の行Noが戻る
def getInsertPointInCnt(lines, header, keywords):
    ip = 0
    #keywordを検索
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[0] == "#":
            flag = 0
            for key in keywords:
                if line.find(key) >= 0:
                    flag = 1
                    break
            if flag == 1:
                i += 1
                if lines[i][0] == "#":
                    i += 1
                #while lines[i][0] == "#" and i < len(lines):
                #    i += 1
                ip =i
                break
        i += 1
    #keywordに該当したか？
    if ip == 0:
        #headerを検索
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len(header)] == header:
                ip = i
                break
            i += 1
    return ip

#
#  remakeGRPID
#    header（!DLOAD等）のGRPIDを連番に修正する
def remakeGRPID(lines, header):
    if header == "!CLOAD" or header == "!DLOAD" or header == "!TEMPERATURE":
        grpId = 1
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[0] == "!":
                lineName = deleteSp(line).split(",")[0]
                if (lineName == "!CLOAD" or lineName == "!DLOAD"
                    or lineName == "!TEMPERATURE"):
                    lines[i] = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                    grpId += 1
            i += 1
    else:
        grpId = 1
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len(header)] == header:
                if deleteSp(lines[i]).split(",")[0] == header:
                    lines[i] = abaqus2fistr.setValName(line, "GRPID="+str(grpId))
                    grpId += 1
            i += 1
    return lines

#
#  getNextHeaderName
#    header名（!SOLVER等）を取得
def getNextHeaderName(lines, i, headerName, itemName):
    while i < len(lines):
        if lines[i][:len(headerName)] == headerName:
            if deleteSp(lines[i]).split(",")[0] == headerName:
                if itemName == "":
                    break
                else:
                    words = itemName.split("=")
                    if abaqus2fistr.getValName(lines[i], words[0]) == words[1]:
                        break
        i += 1
    return i

#
#  skipNextHeader
#
def skipNextHeader(lines, i):
    ans = -1
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            ans = i
            break
        i += 1
    return ans

#
#  deleteHeaderName
#    指定したheaderを削除
def deleteHeaderName(lines, i, header, keyName):
    while i < len(lines):
        i = getNextHeaderName(lines, i, header, keyName)
        if i < len(lines):
            lines = delete1header(lines, i)
    return lines
    """
    i = getNextHeaderName(lines, i, header, keyName)
    if i < len(lines):
        lines = delete1header(lines, i)
    return lines
    """

#
#  delete1header
#    指定のheader位置からheader1ヶ分を削除（「#」または「!」まで
def delete1header(lines, i):
    ip = i
    i += 1
    while i < len(lines):
        if lines[i][0] == "#" or lines[i][0] == "!":
            break
        i += 1
    newLines = lines[:ip] + lines[i:]
    return newLines

#
#  getNextDloadGrpType
#    DLOADの値を取得
#    戻り値　grepTypes:[[grep, bndType, line, i], ...]
def getNextDloadGrpType(lines, i):
    ans = []
    flag = 0
    while i < len(lines):
        line = lines[i]
        if line[:len("!DLOAD")] == "!DLOAD":
            i += 1
            flag = 1
            grpType = []
            while lines[i][0] != "!" and i < len(lines):
                if lines[i][0] != "#":
                    line = deleteSp(lines[i])
                    words = line.split(",")
                    grpType += [[words[0], words[1], line, i]]
                i += 1
        else:
            i += 1
        if flag == 1:
            ans = grpType
            break
    return ans

#
#  getNoGroups
#    allGroupとselGroupからunselectのgroupを返す
def getNoGroups(allGroups, selGroups):
    noSels = []
    for allGroup in allGroups:
        flag = 0
        for selGroup in selGroups:
            if allGroup == selGroup:
                flag = 1
                break
        if flag == 0:
            noSels.append(allGroup)
    return noSels

#
#  deleteSp
#    空白とCRを削除して返す
def deleteSp(line):
    newLine = ""
    i = 0
    for chara in line:
        if chara > " ":
            newLine += chara
        else:
            if i >= 1 and chara == " ":
                if line[i-1] == " " or line[i-1] == ",":
                    pass
                else:
                    newLine += chara
        i += 1 
    return newLine

#
#  deleteValName
#    line内から変数名を削除する
def deleteValName(line, delName):
    words = deleteSp(line).split(",")
    newWords = []
    for word in words:
        valSet = word.split("=")
        if len(valSet) == 2 and valSet[0] == delName:
            pass
        else:
            newWords.append(word)
    newLine = ", ".join(newWords) + ls
    return newLine

